@@ -37,14 +37,14 @@ class RouterCompiler(object):
 
     def compile(self, node):
         phy_node = self.anm['phy'].node(node)
-        ip_node = self.anm['ip'].node(node)
+        ipv4_node = self.anm['ipv4'].node(node)
 
         node.ip.use_ipv4 = phy_node.use_ipv4
         node.ip.use_ipv6 = phy_node.use_ipv6
 
         node.label = naming.network_hostname(phy_node)
         node.input_label = phy_node.id
-        node.loopback = ip_node.loopback
+        node.loopback = ipv4_node.loopback
         node.loopback_subnet = netaddr.IPNetwork(node.loopback)
         node.loopback_subnet.prefixlen = 32
         self.interfaces(node)
@@ -56,17 +56,18 @@ class RouterCompiler(object):
 
     def interfaces(self, node):
         phy_node = self.anm['phy'].node(node)
-        G_ipv4 = self.anm['ip']
-        G_ipv6 = self.anm['ip6']
+        G_ipv4 = self.anm['ipv4']
+        G_ipv6 = self.anm['ipv6']
         node.interfaces = []
         for link in phy_node.edges():
             nidb_edge = self.nidb.edge(link)
 
-            ipv6_address = ipv4_address = ipv4_subnet = None
+            ipv4_cidr = ipv6_address = ipv4_address = ipv4_subnet = None
             if node.ip.use_ipv4:
                 ipv4_link = G_ipv4.edge(link)
                 ipv4_address = ipv4_link.ip_address
                 ipv4_subnet =  ipv4_link.dst.subnet # netmask comes from collision domain on the link
+                ipv4_cidr = address_prefixlen_to_network(ipv4_address, ipv4_subnet.prefixlen)
             if node.ip.use_ipv6:
                 ipv6_link = G_ipv6.edge(link)
                 ipv6_subnet =  ipv6_link.dst.subnet # netmask comes from collision domain on the link
@@ -79,6 +80,7 @@ class RouterCompiler(object):
                     description = "%s to %s" % (link.src, link.dst),
                     ipv4_address = ipv4_address,
                     ipv4_subnet = ipv4_subnet,
+                    ipv4_cidr = ipv4_cidr,
                     ipv6_address = ipv6_address,
                     physical = True,
                     )
@@ -89,7 +91,7 @@ class RouterCompiler(object):
         """Returns OSPF links, also sets process_id
         """
         G_ospf = self.anm['ospf']
-        G_ip = self.anm['ip']
+        G_ipv4 = self.anm['ipv4']
 
         node.ospf.loopback_area = G_ospf.node(node).area
 
@@ -99,7 +101,7 @@ class RouterCompiler(object):
         node.ospf.ospf_links = []
         added_networks = set()
         for link in G_ospf.edges(phy_node):
-            ip_link = G_ip.edge(link)
+            ip_link = G_ipv4.edge(link)
             if not ip_link:
                 #TODO: fix this: due to multi edges from router to same switch cluster
                 continue
@@ -115,14 +117,17 @@ class RouterCompiler(object):
     def bgp(self, node):
         phy_node = self.anm['phy'].node(node)
         G_bgp = self.anm['bgp']
-        G_ip = self.anm['ip']
-        G_ipv6 = self.anm['ip6']
+        G_ipv4 = self.anm['ipv4']
+        G_ipv6 = self.anm['ipv6']
         asn = phy_node.asn # easy reference for cleaner code
         node.asn = asn
-        node.bgp.ipv4_advertise_subnets = G_ip.data.infra_blocks.get(asn) or [] # note: could be none (if single-node AS) - default to empty list
+        node.bgp.ipv4_advertise_subnets = []
+        if node.ip.use_ipv4:
+            node.bgp.ipv4_advertise_subnets = G_ipv4.data.infra_blocks.get(asn) or [] # note: could be none (if single-node AS) - default to empty list
 # put into list
-        #TODO: put advertise subnets from ip6 plugin into a list for consistency with ipv4
-        node.bgp.ipv6_advertise_subnets = [G_ipv6.data.infra_blocks.get(asn)]
+        node.bgp.ipv6_advertise_subnets = []
+        if node.ip.use_ipv6:
+            node.bgp.ipv6_advertise_subnets = G_ipv6.data.infra_blocks.get(asn) or []
          
         node.bgp.ibgp_neighbors = []
         node.bgp.ibgp_rr_clients = []
@@ -132,7 +137,7 @@ class RouterCompiler(object):
         def format_session(session, use_ipv4 = False, use_ipv6 = False):
             neigh = session.dst
             if use_ipv4:
-                neigh_ip = G_ip.node(neigh)
+                neigh_ip = G_ipv4.node(neigh)
             elif use_ipv6:
                 neigh_ip = G_ipv6.node(neigh)
             else:
@@ -157,8 +162,8 @@ class RouterCompiler(object):
                     node.bgp.ibgp_neighbors.append(data)
             else:
                 #TODO: fix this: this is a workaround for Quagga next-hop denied for loopback (even with static route)
-                ip_link = G_ip.edge(session)
-                dst_int_ip = G_ip.edges(ip_link.dst, neigh).next().ip_address #TODO: split this to a helper function
+                ip_link = G_ipv4.edge(session)
+                dst_int_ip = G_ipv4.edges(ip_link.dst, neigh).next().ip_address #TODO: split this to a helper function
                 node.bgp.ebgp_neighbors.append( {
                     'neighbor': neigh.label,
                     'use_ipv4': use_ipv4,
@@ -185,7 +190,7 @@ class QuaggaCompiler(RouterCompiler):
     lo_interface = "lo0:1"
 
     def interfaces(self, node):
-        ip_node = self.anm['ip'].node(node)
+        ipv4_node = self.anm['ipv4'].node(node)
         phy_node = self.anm['phy'].node(node)
         G_ospf = self.anm['ospf']
 
@@ -201,7 +206,7 @@ class QuaggaCompiler(RouterCompiler):
             node.interfaces.append(
                     id = self.lo_interface,
                     description = "Loopback for BGP",
-                    ipv4_address = ip_node.loopback,
+                    ipv4_address = ipv4_node.loopback,
                     ipv4_subnet = node.loopback_subnet
                     )
 
@@ -209,7 +214,7 @@ class QuaggaCompiler(RouterCompiler):
         super(QuaggaCompiler, self).ospf(node)
 
         # add eBGP link subnets
-        G_ip = self.anm['ip']
+        G_ipv4 = self.anm['ipv4']
         G_bgp = self.anm['bgp']
         node.ospf.passive_interfaces = []
         
@@ -219,7 +224,7 @@ class QuaggaCompiler(RouterCompiler):
                     id = nidb_edge.id,
                     )
 
-            ip_link = G_ip.edge(link)
+            ip_link = G_ipv4.edge(link)
             default_ebgp_area = 0
             if not ip_link:
                 #TODO: fix this: due to multi edges from router to same switch cluster
@@ -245,13 +250,14 @@ class IosBaseCompiler(RouterCompiler):
         
     def interfaces(self, node):
 
-        ipv6_address = ipv4_address = ipv4_loopback_subnet = None
+        ipv4_cidr = ipv6_address = ipv4_address = ipv4_loopback_subnet = None
         if node.ip.use_ipv4:
-            ipv4_node = self.anm['ip'].node(node)
+            ipv4_node = self.anm['ipv4'].node(node)
             ipv4_address = ipv4_node.loopback
             ipv4_loopback_subnet = netaddr.IPNetwork("0.0.0.0/32")
+            ipv4_cidr = address_prefixlen_to_network(ipv4_address, ipv4_loopback_subnet.prefixlen)
         if node.ip.use_ipv6:
-            ipv6_node = self.anm['ip6'].node(node)
+            ipv6_node = self.anm['ipv6'].node(node)
             ipv6_address = address_prefixlen_to_network(ipv6_node.loopback, 126)
 
 #TODO: strip out returns from super
@@ -278,6 +284,7 @@ class IosBaseCompiler(RouterCompiler):
                 description = "Loopback",
                 ipv4_address = ipv4_address,
                 ipv4_subnet = ipv4_loopback_subnet,
+                ipv4_cidr = ipv4_cidr,
                 ipv6_address = ipv6_address,
                 isis = is_isis_node,
                 physical = False,
@@ -348,6 +355,29 @@ class Ios2Compiler(IosBaseCompiler):
 
         node.ospf.interfaces = dict(interfaces_by_area)# TODO: workaround for limited wrapping depth, make node.ospf.interfaces grouped by area
 
+class NxOsCompiler(IosBaseCompiler):
+    def interfaces(self, node):
+#need to aggregate areas
+        super(NxOsCompiler, self).interfaces(node)
+
+        for interface in node.interfaces:
+            interface['color'] = "red"
+
+    def ospf(self, node):
+        super(NxOsCompiler, self).ospf(node)
+        for interface in node.interfaces:
+            # get ospf info for this id
+            ospf_link = self.anm['ospf'].edge(interface._edge_id) # find link in OSPF with this ID
+            if ospf_link:
+                area = str(ospf_link.area)
+                interface['ospf'] = {
+                        'area': area,
+                        'process_id': node.ospf.process_id, # from super ospf config
+                        }
+
+        #TODO: configure OSPF on loopback like example
+
+
 # Platform compilers
 class PlatformCompiler(object):
     """Base Platform Compiler"""
@@ -472,11 +502,11 @@ class NetkitCompiler(PlatformCompiler):
         lab_topology.machines = " ".join(alpha_sort(naming.network_hostname(phy_node) 
             for phy_node in subgraph.nodes("is_l3device")))
 
-        G_ip = self.anm['ip']
+        G_ipv4 = self.anm['ipv4']
         lab_topology.config_items = []
         for node in sorted(subgraph.nodes("is_l3device")):
             for edge in node.edges():
-                collision_domain = str(G_ip.edge(edge).dst.subnet).replace("/", ".")
+                collision_domain = str(G_ipv4.edge(edge).dst.subnet).replace("/", ".")
                 numeric_id = edge.id.replace("eth", "") # netkit lab.conf uses 1 instead of eth1
                 lab_topology.config_items.append(
                     device = naming.network_hostname(node),
@@ -514,6 +544,10 @@ class CiscoCompiler(PlatformCompiler):
     def interface_ids_ios(self):
         for x in itertools.count(0):
             yield "GigabitEthernet0/%s" % x
+
+    def interface_ids_nxos(self):
+        for x in itertools.count(0):
+            yield "Ethernet2/%s" % x
             
     def interface_ids_ios2_slot_port(self):
         """Allocate with slot and port iterating
@@ -539,9 +573,9 @@ class CiscoCompiler(PlatformCompiler):
         G_in = self.anm['input']
         G_in_directed = self.anm['input_directed']
         specified_int_names = G_in.data.specified_int_names
+        G_phy = self.anm['phy']
 
         log.info("Compiling Cisco for %s" % self.host)
-        G_phy = self.anm['phy']
         ios_compiler = IosClassicCompiler(self.nidb, self.anm)
         now = datetime.now()
         if settings['Compiler']['Cisco']['timestamp']:
@@ -594,6 +628,30 @@ class CiscoCompiler(PlatformCompiler):
 
             ios2_compiler.compile(nidb_node)
 
+            
+        nxos_compiler = NxOsCompiler(self.nidb, self.anm)
+        for phy_node in G_phy.nodes('is_router', host = self.host, syntax='nx_os'):
+            nidb_node = self.nidb.node(phy_node)
+            #nidb_node.render.base = "templates/ios2"
+            #nidb_node.render.base_dst_folder = "rendered/%s/%s/%s" % (self.host, "cisco", folder_name)
+            nidb_node.render.template = "templates/nx_os.mako"
+            if self.to_memory:
+                nidb_node.render.to_memory = True
+            else:
+                nidb_node.render.dst_folder = dst_folder
+                nidb_node.render.dst_file = "%s.conf" % naming.network_hostname(phy_node)
+
+            # Assign interfaces
+            int_ids = self.interface_ids_nxos()
+            for edge in sorted(self.nidb.edges(nidb_node), key = edge_id_numeric):
+                if specified_int_names:
+                    directed_edge = G_in_directed.edge(edge)
+                    edge.id = directed_edge.name
+                else:
+                    edge.id = int_ids.next()
+
+            nxos_compiler.compile(nidb_node)
+
 
         other_nodes = [phy_node for phy_node in G_phy.nodes('is_router', host = self.host)
                 if phy_node.syntax not in ("ios", "ios2")]
