@@ -10,12 +10,16 @@ except ImportError:
     log.warning("Deployment requires Exscript: "
     "pip install https://github.com/knipknap/exscript/tarball/master")
 
-def package(src_dir, target):
+def deploy(host, username, dst_folder):
+    tar_file = package(dst_folder)
+    transfer(host, username, tar_file)
+    extract(host, username, tar_file, dst_folder)
+
+def package(src_dir, target = "netkit_lab"):
     log.info("Packaging %s" % src_dir)
     import tarfile
     import os
     tar_filename = "%s.tar.gz" % target
-#time.strftime("%Y%m%d_%H%M", time.localtime())
     tar = tarfile.open(os.path.join(tar_filename), "w:gz")
     tar.add(src_dir)
     tar.close()
@@ -28,11 +32,7 @@ def transfer(host, username, local, remote = None, key_filename = None):
         remote = local # same filename
     import paramiko
     ssh = paramiko.SSHClient()
-    ssh.set_missing_host_key_policy(
-        paramiko.AutoAddPolicy())
-# handle key_filename of '' (empty string)
-    if key_filename and not len(key_filename): # TODO: See if this is needed
-        key_filename = None
+    ssh.set_missing_host_key_policy( paramiko.AutoAddPolicy())
     if key_filename:
         log.debug("Connecting to %s with %s and key %s" % (host, username, key_filename))
         ssh.connect(host, username = username, key_filename = key_filename)
@@ -58,21 +58,8 @@ def extract(host, username, tar_file, cd_dir, timeout = 30, key_filename = None,
 
     messaging = ank_messaging.AnkMessaging()
 
-    use_rabbitmq = config.settings['Rabbitmq']['active']
-    if use_rabbitmq:
-        import pika
-        import json
-        pika_host = config.settings['Rabbitmq']['server']
-        www_connection = pika.BlockingConnection(pika.ConnectionParameters(
-                host = pika_host))
-        www_channel = www_connection.channel()
-        www_channel.exchange_declare(exchange='www',
-                type='direct')
-
-
     def starting_host(protocol, index, data):
         m = re.search('\\"(\S+)\\"', data.group(index))
-#TODO: reverse lookup from foldername to the canonical id of device
         if m:
             hostname = m.group(1)
             log.info(data.group(index)) #TODO: use regex to strip out just the machine name
@@ -80,12 +67,12 @@ def extract(host, username, tar_file, cd_dir, timeout = 30, key_filename = None,
             messaging.publish_json(body)
 
     def lab_started(protocol, index, data):
+        log.info("Lab started on %s" % host)
         body = {"lab started": host}
         messaging.publish_json(body)
 
     def make_not_found(protocol, index, data):
         log.warning("Make not installed on remote host %s. Please install make and retry." % host)
-#TODO: raise exception here, catch in the start script
         return
 
     def start_lab(thread, host, conn):
@@ -109,16 +96,9 @@ def extract(host, username, tar_file, cd_dir, timeout = 30, key_filename = None,
         except InvalidCommandException, error:
             if "already running" in str(error):
                 time.sleep(1)
-                #print "Already Running" #TODO: handle appropriately
-                #print "Halting previous lab"
-                #conn.execute("vclean -K")
-                #print "Halted previous lab"
-                #conn.execute("vstart taptunnelvm --con0=none --eth0=tap,172.16.0.1,172.16.0.2") # TODO: don't hardcode this
-                #print "Starting lab"
                 conn.execute(start_command)
         first_match(conn, r'^The lab has been started')
         conn.send("exit")
-#TODO: need to capture and handle tap startup
 
     if key_filename:
         key = PrivateKey.from_file(key_filename)
@@ -129,4 +109,4 @@ def extract(host, username, tar_file, cd_dir, timeout = 30, key_filename = None,
         accounts = [Account(username)] 
 
     hosts = ['ssh://%s' % host]
-    start(accounts, hosts, start_lab, verbose = 2)
+    start(accounts, hosts, start_lab, verbose = verbosity)
